@@ -20,7 +20,7 @@ from pants.base.cache_manager import (InvalidationCacheManager, InvalidationChec
 from pants.base.config import Config
 from pants.base.exceptions import TaskError
 from pants.base.worker_pool import Work
-from pants.base.workunit import WorkUnit
+from pants.cache.artifact_cache import call_insert, call_use_cached_files, UnreadableArtifact
 from pants.cache.cache_setup import create_artifact_cache
 from pants.cache.read_write_artifact_cache import ReadWriteArtifactCache
 from pants.reporting.reporting_utils import items_to_report_element
@@ -32,7 +32,7 @@ class TaskBase(AbstractClass):
 
   Provides the base lifecycle methods that allow a task to interact with the command line, other
   tasks and the user.  The lifecycle is linear and run via the following sequence:
-  1. setup_parser - expose command line flags
+  1. register_options - declare options configurable via cmd-line flag or config file.
   2. __init__ - distill configuration into the information needed to execute
   3. prepare - request any products needed from goal dependencies
 
@@ -47,15 +47,37 @@ class TaskBase(AbstractClass):
   of the helpers.  Ideally console tasks don't inherit a workdir, invalidator or build cache for
   example.
   """
+  # The scope for this task's options. Will be set (on a synthetic subclass) during registration.
+  options_scope = None
 
   @classmethod
-  def setup_parser(cls, option_group, args, mkflag):
-    """Set up the cmd-line parser.
+  def known_scopes(cls):
+    """Yields all known scopes under this task (usually just its own.)"""
+    yield cls.options_scope
 
-    Subclasses can add flags to the pants command line using the given option group.
-    Flag names should be created with mkflag([name]) to ensure flags are properly name-spaced
-    amongst other tasks.
+  @classmethod
+  def register_options_on_scope(cls, options):
+    """Trigger registration of this task's options.
+
+    Subclasses should not generally need to override this method.
     """
+    def register(*args, **kwargs):
+      options.register(cls.options_scope, *args, **kwargs)
+    register.bootstrap = options.bootstrap_option_values()
+    cls.register_options(register)
+
+  @classmethod
+  def register_options(cls, register):
+    """Set up the new options system.
+
+    Subclasses may override and call register(*args, **kwargs) with argparse arguments
+    to register options.
+    """
+
+  @classmethod
+  def supports_passthru_args(cls):
+    """Subclasses may override to indicate that they can use passthru args."""
+    return False
 
   def __init__(self, context, workdir):
     """Subclass __init__ methods, if defined, *must* follow this idiom:
@@ -76,13 +98,25 @@ class TaskBase(AbstractClass):
     self._artifact_cache = None
     self._artifact_cache_setup_lock = threading.Lock()
 
-    default_invalidator_root = os.path.join(self.context.config.getdefault('pants_workdir'),
-                                            'build_invalidator')
+    self._cache_key_errors = set()
+
+    default_invalidator_root = os.path.join(
+      self.context.new_options.for_global_scope().pants_workdir, 'build_invalidator')
     suffix_type = self.__class__.__name__
     self._build_invalidator_dir = os.path.join(
         context.config.get('tasks', 'build_invalidator', default=default_invalidator_root),
         suffix_type)
 
+  def get_options(self):
+    """Returns the option values for this task's scope."""
+    return self.context.new_options.for_scope(self.options_scope)
+
+  def get_passthru_args(self):
+    if not self.supports_passthru_args():
+      raise TaskError('{0} Does not support passthru args.'.format(self.__class__.__name__))
+    else:
+      return self.context.new_options.passthru_args_for_scope(self.options_scope)
+
   def prepare(self, round_manager):
     """Prepares a task for execution.
 
@@ -94,6 +128,11 @@ class TaskBase(AbstractClass):
 
   @property
   def workdir(self):
+    """A scratch-space for this task that will be deleted by `clean-all`.
+
+    It's not guaranteed that the workdir exists, just that no other task has been given this
+    workdir path to use.
+    """
     return self._workdir
 
   def setup_artifact_cache_from_config(self, config_section=None):
@@ -119,9 +158,16 @@ class TaskBase(AbstractClass):
 
   def _create_artifact_cache(self, spec, action):
     if len(spec) > 0:
-      pants_workdir = self.context.config.getdefault('pants_workdir')
+      pants_workdir = self.context.new_options.for_global_scope().pants_workdir
+      compression = self.context.config.getint('cache', 'compression', default=5)
       my_name = self.__class__.__name__
-      return create_artifact_cache(self.context.log, pants_workdir, spec, my_name, action)
+      return create_artifact_cache(
+        log=self.context.log,
+        artifact_root=pants_workdir,
+        spec=spec,
+        task_name=my_name,
+        compression=compression,
+        action=action)
     else:
       return None
 
@@ -135,10 +181,10 @@ class TaskBase(AbstractClass):
       return self._artifact_cache
 
   def artifact_cache_reads_enabled(self):
-    return bool(self._read_artifact_cache_spec) and self.context.options.read_from_artifact_cache
+    return bool(self._read_artifact_cache_spec) and self.get_options().read_from_artifact_cache
 
   def artifact_cache_writes_enabled(self):
-    return bool(self._write_artifact_cache_spec) and self.context.options.write_to_artifact_cache
+    return bool(self._write_artifact_cache_spec) and self.get_options().write_to_artifact_cache
 
   @classmethod
   def product_types(self):
@@ -254,11 +300,12 @@ class TaskBase(AbstractClass):
 
     if not silent:
       targets = []
-      payloads = []
       num_invalid_partitions = len(invalidation_check.invalid_vts_partitioned)
       for vt in invalidation_check.invalid_vts_partitioned:
         targets.extend(vt.targets)
-        payloads.extend(vt.cache_key.payloads)
+
+      payloads = [t.payload for t in targets]
+
       if len(targets):
         msg_elements = ['Invalidated ',
                         items_to_report_element([t.address.reference() for t in targets], 'target')]
@@ -303,14 +350,18 @@ class TaskBase(AbstractClass):
     cached_vts = []
     uncached_vts = OrderedSet(vts)
 
-    with self.context.new_workunit(name='check', labels=[WorkUnit.MULTITOOL]) as parent:
-      res = self.context.submit_foreground_work_and_wait(
-        Work(lambda vt: bool(self.get_artifact_cache().use_cached_files(vt.cache_key)),
-             [(vt, ) for vt in vts], 'fetch'), workunit_parent=parent)
+    cache = self.get_artifact_cache()
+    items = [(cache, vt.cache_key) for vt in vts]
+
+    res = self.context.subproc_map(call_use_cached_files, items)
+
     for vt, was_in_cache in zip(vts, res):
       if was_in_cache:
         cached_vts.append(vt)
         uncached_vts.discard(vt)
+      elif isinstance(was_in_cache, UnreadableArtifact):
+        self._cache_key_errors.update(was_in_cache.key)
+
     # Note that while the input vts may represent multiple targets (for tasks that overrride
     # check_artifact_cache_for), the ones we return must represent single targets.
     def flatten(vts):
@@ -350,11 +401,16 @@ class TaskBase(AbstractClass):
       for vts, _ in vts_artifactfiles_pairs:
         targets.update(vts.targets)
       self._report_targets('Caching artifacts for ', list(targets), '.')
+
+      always_overwrite = self.get_options().overwrite_cache_artifacts
+
       # Cache the artifacts.
       args_tuples = []
       for vts, artifactfiles in vts_artifactfiles_pairs:
-        args_tuples.append((vts.cache_key, artifactfiles))
-      return Work(lambda *args: cache.insert(*args), args_tuples, 'insert')
+        overwrite = always_overwrite or vts.cache_key in self._cache_key_errors
+        args_tuples.append((cache, vts.cache_key, artifactfiles, overwrite))
+
+      return Work(lambda x: self.context.subproc_map(call_insert, x), [(args_tuples,)], 'insert')
     else:
       return None
 
