@@ -13,6 +13,7 @@ import shutil
 from twitter.common.collections.orderedset import OrderedSet
 
 from pants import binary_util
+from pants.backend.jvm.jvm_debug_config import JvmDebugConfig
 from pants.backend.jvm.targets.scala_library import ScalaLibrary
 from pants.backend.jvm.tasks.checkstyle import Checkstyle
 from pants.backend.jvm.tasks.jvm_binary_task import JvmBinaryTask
@@ -22,7 +23,7 @@ from pants.base.exceptions import TaskError
 from pants.base.source_root import SourceRoot
 from pants.base.target import Target
 from pants.goal.goal import Goal
-from pants.util.dirutil import safe_mkdir
+from pants.util.dirutil import safe_mkdir, safe_walk
 
 logger = logging.getLogger(__name__)
 
@@ -43,76 +44,49 @@ def is_java(target):
 
 # XXX(pl): JVM hairball violator (or is it just JVM specific?)
 class IdeGen(JvmBinaryTask, JvmToolTaskMixin):
+
   @classmethod
-  def setup_parser(cls, option_group, args, mkflag):
-    super(IdeGen, cls).setup_parser(option_group, args, mkflag)
-
-    option_group.add_option(mkflag("project-name"), dest="ide_gen_project_name", default="project",
-                            help="[%default] Specifies the name to use for the generated project.")
-
-    gen_dir = mkflag("project-dir")
-    option_group.add_option(gen_dir, dest="ide_gen_project_dir",
-                            help="[%default] Specifies the directory to output the generated "
-                                "project files to.")
-    option_group.add_option(mkflag("project-cwd"), dest="ide_gen_project_cwd",
-                            help="[%%default] Specifies the directory the generated project should "
-                                 "use as the cwd for processes it launches.  Note that specifying "
-                                 "this trumps %s and not all project related files will be stored "
-                                 "there." % gen_dir)
-
-    option_group.add_option(mkflag("intransitive"), default=False,
-                            action="store_true", dest='ide_gen_intransitive',
-                            help="Limits the sources included in the generated project to just "
-                                 "those owned by the targets specified on the command line")
-
-    option_group.add_option(mkflag("python"), mkflag("python", negate=True), default=False,
-                            action="callback", callback=mkflag.set_bool, dest='ide_gen_python',
-                            help="[%default] Adds python support to the generated project "
-                                 "configuration.")
-
-    option_group.add_option(mkflag("java"), mkflag("java", negate=True), default=True,
-                            action="callback", callback=mkflag.set_bool, dest='ide_gen_java',
-                            help="[%default] Includes java sources in the project; otherwise "
-                                 "compiles them and adds them to the project classpath.")
-    java_language_level = mkflag("java-language-level")
-
-    option_group.add_option(java_language_level, default=7,
-                            dest="ide_gen_java_language_level", type="int",
-                            help="[%default] Sets the java language and jdk used to compile the "
-                                 "project's java sources.")
-    option_group.add_option(mkflag("java-jdk-name"), default=None,
-                            dest="ide_gen_java_jdk",
-                            help="Sets the jdk used to compile the project's java sources. If "
-                                 "unset the default jdk name for the "
-                                 "%s is used." % java_language_level)
-
-    option_group.add_option(mkflag("scala"), mkflag("scala", negate=True), default=True,
-                            action="callback", callback=mkflag.set_bool, dest='ide_gen_scala',
-                            help="[%default] Includes scala sources in the project; otherwise "
-                                 "compiles them and adds them to the project classpath.")
-
-    option_group.add_option(mkflag("use-source-root"), mkflag("use-source-root", negate=True),
-                            default=False, action="callback", callback=mkflag.set_bool,
-                            dest='ide_gen_use_source_root',
-                            help="[%default] Use source_root() settings to collapse source"
-                                 "paths in project and determine which paths are used for "
-                                 "tests.  This is usually what you want if your repo uses "
-                                 "a maven style directory layout.")
-    option_group.add_option(mkflag("infer-test-from-siblings"),
-                            mkflag("infer-test-from-siblings", negate=True),
-                            default=True, action="callback", callback=mkflag.set_bool,
-                            dest='ide_infer_test_from_siblings',
-                            help="[%default] When determining if a path should be added to the "
-                                 "IDE, check to see if any of its sibling source_root() entries "
-                                 "define test targets.  This is usually what you want so that "
-                                 "resource directories under test source roots are picked up as "
-                                 "test paths.")
+  def register_options(cls, register):
+    super(IdeGen, cls).register_options(register)
+    register('--project-name', default='project',
+             help='Specifies the name to use for the generated project.')
+    register('--project-dir',
+             help='Specifies the directory to output the generated project files to.')
+    register('--project-cwd',
+             help='Specifies the directory the generated project should use as the cwd for '
+                  'processes it launches.  Note that specifying this trumps --{0}-project-dir '
+                  'and not all project related files will be stored there.'
+                  .format(cls.options_scope))
+    register('--intransitive', action='store_true', default=False,
+             help='Limits the sources included in the generated project to just '
+                  'those owned by the targets specified on the command line.')
+    register('--python', action='store_true', default=False,
+             help='Adds python support to the generated project configuration.')
+    register('--java', action='store_true', default=True,
+             help='Includes java sources in the project; otherwise compiles them and adds them '
+                   'to the project classpath.')
+    register('--java-language-level', type=int, default=7,
+             help='Sets the java language and jdk used to compile the project\'s java sources.')
+    register('--java-jdk-name', default=None,
+             help='Sets the jdk used to compile the project\'s java sources. If unset the default '
+                  'jdk name for the --java-language-level is used')
+    register('--scala', action='store_true', default=True,
+             help='Includes scala sources in the project; otherwise compiles them and adds them '
+                  'to the project classpath.')
+    register('--use-source-root', action='store_true', default=False,
+             help='Use source_root() settings to collapse sourcepaths in project and determine '
+                  'which paths are used for tests.  This is usually what you want if your repo '
+                  ' uses a maven style directory layout.')
+    register('--infer-test-from-siblings', action='store_true',
+             help='When determining if a path should be added to the IDE, check to see if any of '
+                  'its sibling source_root() entries define test targets.  This is usually what '
+                  'you want so that resource directories under test source roots are picked up as '
+                  'test paths.')
 
   class Error(TaskError):
     """IdeGen Error."""
 
-  # XXX(pl): What is up with this subclassing?  It seems completely wrong.
-  class TargetUtil(Target):
+  class TargetUtil(object):
     def __init__(self, context):
       self.context = context
 
@@ -124,56 +98,58 @@ class IdeGen(JvmBinaryTask, JvmToolTaskMixin):
       return set(self.context.address_mapper.addresses_in_spec_path(buildfile.spec_path))
 
     def get(self, address):
-      self.context.build_graph.inject_address(address)
+      self.context.build_graph.inject_address_closure(address)
       return self.context.build_graph.get_target(address)
 
   def __init__(self, *args, **kwargs):
     super(IdeGen, self).__init__(*args, **kwargs)
 
-    self.project_name = self.context.options.ide_gen_project_name
-    self.python = self.context.options.ide_gen_python
-    self.skip_java = not self.context.options.ide_gen_java
-    self.skip_scala = not self.context.options.ide_gen_scala
-    self.use_source_root = self.context.options.ide_gen_use_source_root
+    self.project_name = self.get_options().project_name
+    self.python = self.get_options().python
+    self.skip_java = not self.get_options().java
+    self.skip_scala = not self.get_options().scala
+    self.use_source_root = self.get_options().use_source_root
 
-    self.java_language_level = self.context.options.ide_gen_java_language_level
-    if self.context.options.ide_gen_java_jdk:
-      self.java_jdk = self.context.options.ide_gen_java_jdk
+    self.java_language_level = self.get_options().java_language_level
+    if self.get_options().java_jdk_name:
+      self.java_jdk = self.get_options().java_jdk_name
     else:
       self.java_jdk = '1.%d' % self.java_language_level
 
     # Always tack on the project name to the work dir so each project gets its own linked jars,
     # etc. See https://github.com/pantsbuild/pants/issues/564
-    if self.context.options.ide_gen_project_dir:
+    if self.get_options().project_dir:
       self.gen_project_workdir = os.path.abspath(
-        os.path.join(self.context.options.ide_gen_project_dir, self.project_name))
+        os.path.join(self.get_options().project_dir, self.project_name))
     else:
       self.gen_project_workdir = os.path.abspath(
         os.path.join(self.workdir, self.__class__.__name__, self.project_name))
 
     self.cwd = (
-      os.path.abspath(self.context.options.ide_gen_project_cwd) if
-      self.context.options.ide_gen_project_cwd else self.gen_project_workdir
+      os.path.abspath(self.get_options().project_cwd) if
+      self.get_options().project_cwd else self.gen_project_workdir
     )
 
-    self.intransitive = self.context.options.ide_gen_intransitive
+    self.intransitive = self.get_options().intransitive
 
-    self.checkstyle_suppression_files = self.context.config.getdefault(
-      'checkstyle_suppression_files', type=list, default=[]
-    )
-    self.debug_port = self.context.config.getint('ide', 'debug_port', default=5005)
+    # Everywhere else, debug_port is specified in the 'jvm' section. Use that as a default if none
+    # is specified in the 'ide' section.
+    jvm_config_debug_port = JvmDebugConfig.debug_port(self.context.config)
+    self.debug_port = self.context.config.getint('ide', 'debug_port', default=jvm_config_debug_port)
 
     self.checkstyle_bootstrap_key = 'checkstyle'
-    checkstyle = self.context.config.getlist('checkstyle', 'bootstrap-tools',
-                                             default=['//:twitter-checkstyle'])
-    self.register_jvm_tool(self.checkstyle_bootstrap_key, checkstyle)
+    self.register_jvm_tool_from_config(self.checkstyle_bootstrap_key, self.context.config,
+                                       ini_section='compile.checkstyle',
+                                       ini_key='bootstrap_tools',
+                                       default=['//:checkstyle'])
 
     self.scalac_bootstrap_key = None
     if not self.skip_scala:
       self.scalac_bootstrap_key = 'scalac'
-      scalac = self.context.config.getlist('scala-compile', 'compile-bootstrap-tools',
-                                           default=['//:scala-compiler-2.9.3'])
-      self.register_jvm_tool(self.scalac_bootstrap_key, scalac)
+      self.register_jvm_tool_from_config(self.scalac_bootstrap_key, self.context.config,
+                                         ini_section='scala-compile',
+                                         ini_key='compile-bootstrap-tools',
+                                         default=['//:scala-compiler-2.9.3'])
 
   def prepare(self, round_manager):
     if self.python:
@@ -188,12 +164,11 @@ class IdeGen(JvmBinaryTask, JvmToolTaskMixin):
   def _prepare_project(self):
     targets, self._project = self.configure_project(
         self.context.targets(),
-        self.checkstyle_suppression_files,
         self.debug_port)
 
     self.configure_compile_context(targets)
 
-  def configure_project(self, targets, checkstyle_suppression_files, debug_port):
+  def configure_project(self, targets, debug_port):
     jvm_targets = [t for t in targets if t.has_label('jvm') or t.has_label('java')]
     if self.intransitive:
       jvm_targets = set(self.context.target_roots).intersection(jvm_targets)
@@ -203,7 +178,6 @@ class IdeGen(JvmBinaryTask, JvmToolTaskMixin):
                       self.skip_scala,
                       self.use_source_root,
                       get_buildroot(),
-                      checkstyle_suppression_files,
                       debug_port,
                       jvm_targets,
                       not self.intransitive,
@@ -463,8 +437,7 @@ class Project(object):
     return collapsed_source_sets
 
   def __init__(self, name, has_python, skip_java, skip_scala, use_source_root, root_dir,
-               checkstyle_suppression_files, debug_port, targets, transitive, workunit_factory,
-               target_util):
+               debug_port, targets, transitive, workunit_factory, target_util):
     """Creates a new, unconfigured, Project based at root_dir and comprised of the sources visible
     to the given targets."""
 
@@ -487,7 +460,6 @@ class Project(object):
     self.has_scala = False
     self.has_tests = False
 
-    self.checkstyle_suppression_files = checkstyle_suppression_files  # Absolute paths.
     self.debug_port = debug_port
 
     self.internal_jars = OrderedSet()
@@ -515,7 +487,7 @@ class Project(object):
     targeted = set()
 
     def relative_sources(target):
-      sources = target.payload.sources_relative_to_buildroot()
+      sources = target.payload.sources.relative_to_buildroot()
       return [os.path.relpath(source, target.target_base) for source in sources]
 
     def source_target(target):
@@ -646,7 +618,7 @@ class Project(object):
     for source_set in self.sources:
       paths = set()
       source_base = os.path.join(self.root_dir, source_set.source_base)
-      for root, dirs, _ in os.walk(os.path.join(source_base, source_set.path)):
+      for root, dirs, _ in safe_walk(os.path.join(source_base, source_set.path)):
         if dirs:
           paths.update([os.path.join(root, directory) for directory in dirs])
       unused_children = paths - targeted
